#if defined __APPLE__
#  include "SamplesMacOS.h"
#else
#  include "sample.h"
#endif

#include "sample_common.h"
#include "sprk.h"

#ifdef SPRK_PUBLISH_SUPPORTED
#  include "sprk_exchange.h"
#  include "sprk_publish.h"
#endif

#include <iostream>

using namespace HPS;

void Sample::Exchange_Publish(WindowKey wk, Canvas canvas, View view, Model model, SegmentKey modelKey)
{
#ifdef SPRK_PUBLISH_SUPPORTED
	try
	{

	/* Sample: Exchange_Publish
		 * This sample will show you how to:
		 * - Load an Exchange file and check for IO exceptions
		 * - Output a 3D PDF file through Publish and check for IO exceptions
		 *
		 * Read more about the topics covered in this sample in our Programming Guide
		 * - Integration with HOOPS Exchange
		 * - Integration with HOOPS Publish */


	HPS::TextKit textKit;
	textKit.SetSize(0.02f, Text::SizeUnits::SubscreenRelative);
	PointArray pointArray(3);
	pointArray[0] = HPS::Point(-0.95f, -0.70f, 10);
	pointArray[1] = HPS::Point(0.95f, -0.70f, 10);
	pointArray[2] = HPS::Point(0, -1, 10);
	textKit.SetRegion(pointArray, Text::RegionAlignment::Center, Text::RegionFitting::Left, false, false, true);
	textKit.SetBackground(true, "rounded box");
	textKit.SetAlignment(HPS::Text::Alignment::BottomLeft);
	HPS::MaterialMappingKit matMapKit = HPS::MaterialMappingKit();
	matMapKit.SetFaceColor(RGBAColor(1, 1, 1));
	matMapKit.SetEdgeColor(RGBAColor(0, 0, 0));

	/*The next block of code show you how to set the Camera properties to something more familiar
	 * 1. Set the camera to it's default properties
	 * 2. Set the world handedness to "Left" for world positions akin to the Cartesian coordinates */

	view.GetSegmentKey().SetCamera(CameraKit::GetDefault());
	view.GetSegmentKey().GetDrawingAttributeControl().SetWorldHandedness(Drawing::Handedness::Left);

	/* This next block of code shows you how to load an Exchange file
	 * 1. Create an ImportOptionsKit and set the segment in which you want the file to be loaded
	 * 2. Use the Exchange::File::Import method to load the desired file, with the options you specified
	 * 3. Since this operation happens on a separate thread, wait for it to complete before proceeding
	 * 4. Use a try-catch block to capture any Stream IO exceptions
	 * 5. Obtain an Exchange::CADModel object through the Exchange::ImportNotifier
	 * 6. Obtain the new View from the CADModel object. Attach the new view to the Canvas and delete the old View
	 *		You will need to re-insert geometry that was in the old View, such as lights.
	 *		You will also need to re-apply any operators which were active on the old view.*/
	UTF8 message;

	Exchange::ImportNotifier exchangeNotifier;
	try {
		UTF8 file_in = INPUT_FOLDER + "/Crank-Instructions.prc";
		exchangeNotifier = Exchange::File::Import(static_cast<const char*>(file_in), Exchange::ImportOptionsKit());
		exchangeNotifier.Wait();
	}

	catch (IOException ioe) {
		HPS::SegmentKey catchKey = modelKey.Subsegment();
		message = "Exchange Import Unsuccessful";

		view.GetSegmentKey().GetVisibilityControl().SetEdges(true);
		view.GetSegmentKey().SetMaterialMapping(matMapKit);

		textKit.SetText(message);
		catchKey.InsertText(textKit);
		view.FitWorld();
		view.Update();
	}

	CADModel modelFile = exchangeNotifier.GetCADModel();
	view.Delete();
	view = modelFile.ActivateDefaultCapture();
	canvas.AttachViewAsLayout(view);
	modelKey.InsertDistantLight(Vector(1, 1, 0));
	view.FitWorld();

	HPS::SegmentKey results_segment = view.GetSegmentKey().Down("", true);
	results_segment.GetVisibilityControl().SetEdges(true);
	view.GetSegmentKey().SetMaterialMapping(matMapKit);


	/* This next block of code shows you how to output a 3D PDF file
	 * 1. Create SprocketPath object, by passing the keys to the canvas, layout, view and model to it
	 * 2. Use the Publish::File::Export method to output a 3D PDF file */
	if (exchangeNotifier.Status() == IOResult::Success)
	{
		bool publishExportSuccess = true;
		UTF8 file_out = OUTPUT_FOLDER + "/exchange_publish_sample.pdf";

		try {
			SprocketPath sprkPath(canvas, canvas.GetAttachedLayout(),
				canvas.GetAttachedLayout().GetAttachedView(),
				canvas.GetAttachedLayout().GetAttachedView().GetAttachedModel());
			Publish::File::ExportPDF(sprkPath.GetKeyPath(), static_cast<const char*>(file_out), Publish::ExportOptionsKit());
		}

		catch (IOException ioe) {
			message = "Publish Export Unsuccessful";
			publishExportSuccess = false;

			view.GetSegmentKey().GetVisibilityControl().SetEdges(true);
			view.GetSegmentKey().SetMaterialMapping(matMapKit);

			textKit.SetText(message);
			results_segment.InsertText(textKit);
			view.FitWorld();
			view.Update();
		}

		if (publishExportSuccess)
		{
			message =
				"Import and Export completed successfully.\n"
				"You can find the 3D PDF here:\n   " +
				file_out;

			textKit.SetText(message);
			results_segment.InsertText(textKit);
			view.GetSegmentKey().GetCameraControl().SetTarget(Point(-250, -100, 0));
			view.Update();
		}

	}

	}
	catch(IOException e)
	{
		std::cout << e.what() << std::endl;
	}
#else
	model.GetSegmentKey().InsertText(Point(0, 0, 0), "HOOPS Publish is not supported on this platform");
	model.GetSegmentKey().GetVisibilityControl().SetText(true);
	model.GetSegmentKey().GetTextAttributeControl().SetAlignment(HPS::Text::Alignment::Center);
#endif
}
